from citrination_client.models.columns.base import BaseColumn


class FormulationColumn(BaseColumn):
    """
    A formulation column configuration for a data view.
    Parameterized with the basic column options.
    """

    TYPE = "Formulation"
